import pandas as pd
from sklearn.ensemble import ExtraTreesRegressor
from xgboost import XGBRegressor
from lightgbm import LGBMRegressor
from catboost import CatBoostRegressor
from sklearn.ensemble import RandomForestRegressor
from sklearn.metrics import mean_squared_error

from prediccion_utils import read_dataset
import numpy as np
from skopt import BayesSearchCV
from scipy.stats import loguniform

##### Busqueda de modelos #####

df_total , _, _ = read_dataset('../data/UH_2023_TRAIN.txt')

df_train = df_total[df_total["CAMPAÑA"] < 21]
df_test = df_total[df_total["CAMPAÑA"] == 21]

y_train = df_train.pop("PRODUCCION")
y_test = df_test.pop("PRODUCCION")


X_train = df_train[["SUPERFICIE","MODO","VARIEDAD_32","VARIEDAD_87", "VARIEDAD_17",
"ALTITUD_MAX","ALTITUD_MIN","PRODUCCION","PRODUCCION_MEDIA"]].values


X_test = df_test[["SUPERFICIE","MODO","VARIEDAD_32","VARIEDAD_87", "VARIEDAD_17",
"ALTITUD_MAX","ALTITUD_MIN","PRODUCCION","PRODUCCION_MEDIA"]].values


# BayesSearchCV
parameters_RandomForest = { "n_estimators":[int(x) for x in np.linspace(start=200, stop=2000, num=10)],
                           "max_features": ["auto","sqrt"],
                           "max_depth": [int(x) for x in np.linspace(10,110, num=11)]+[None],
                           "min_samples_split": [2,5,10],
                           "min_samples_leaf":[1,2,4],
                           "bootstrap":[True, False]}



parameters_xgboost = {  "booster": ["gbtree"],
                    "n_estimators": list(range(10, 501, 10)),  # 10, 20, ... 500
                    "eta": loguniform.rvs(1e-4, 0.5, size=100),
                    "min_child_weight": loguniform.rvs(1e-3, 1.0, size=100),
                    "max_depth": list(range(3, 50)),
                    "gamma": loguniform.rvs(1e-4, 2, size=100),
                    "subsample": loguniform.rvs(0.1, 1, size=100),
                    "colsample_bytree": loguniform.rvs(0.1, 1, size=100),
                    "lambda": loguniform.rvs(1e-5, 5.0, size=100),
                    "alpha": loguniform.rvs(1e-5, 5.0, size=100),
                    "eval_metric": ["mae"]}


parameters_lightgbm = { "objective": ["regression"],
                        "boosting_type": ["gbdt", "dart", "rf"],
                        "num_leaves": list(range(10, 151, 5)),
                        "max_depth": [-1] + list(range(3, 20)),
                        "learning_rate": loguniform.rvs(1e-5, 0.5, size=100),
                        "n_estimators": list(range(10, 1001, 10)),  # 10, 20, ... 500
                        "min_split_gain": loguniform.rvs(1e-7, 1.0, size=100),
                        "min_child_weight": loguniform.rvs(1e-2, 5.0, size=100),
                        "min_child_samples": list(range(2, 51)),
                        "subsample": loguniform.rvs(0.25, 0.999, size=100),
                        "subsample_freq": list(range(1, 101, 5)),
                        "colsample_bytree": loguniform.rvs(0.25, 1, size=100),
                        "reg_lambda": loguniform.rvs(1e-5, 5.0, size=100),
                        "reg_alpha": loguniform.rvs(1e-5, 5.0, size=100),
                        "metric": ["mae"]}


parameters_catboost = { "iterations": list(range(10, 501, 10)),
                        "learning_rate": loguniform.rvs(1e-5, 0.5, size=100),
                        "l2_leaf_reg": loguniform.rvs(1e-5, 5.0, size=100),
                        "bootstrap_type": ["Bayesian"],
                        "bagging_temperature": loguniform.rvs(1e-2, 10.0, size=100),
                        "depth": list(range(2, 17)),
                        "grow_policy": ["SymmetricTree", "Depthwise", "Lossguide"],
                        "min_data_in_leaf": list(range(1, 10)),
                        "loss_function": ["RMSE"],
                        "logging_level": ["Silent"]}


model_RandomForest = BayesSearchCV(RandomForestRegressor(n_jobs=-1,random_state=0), search_spaces=parameters_RandomForest, n_iter=30, cv=5, verbose=2)
model_xgboost = BayesSearchCV(XGBRegressor(n_jobs=-1,random_state=0), search_spaces=parameters_xgboost, n_iter=30, cv=5, verbose=2)
model_lightgbm = BayesSearchCV(LGBMRegressor(n_jobs=-1,random_state=0), search_spaces=parameters_lightgbm, n_iter=30, cv=5, verbose=2)
model_catbootst = BayesSearchCV(CatBoostRegressor(thread_count=-1,random_state=0), search_spaces=parameters_catboost, n_iter=30, cv=5, verbose=2)


model_RandomForest.fit(X_train, y_train)
model_xgboost.fit(X_train, y_train)
model_lightgbm.fit(X_train, y_train)
model_catbootst.fit(X_train, y_train)



# Guardamos los resultados en un csv para obtener el mejor modelo
df_RandomForest = pd.DataFrame(model_RandomForest.cv_results_)
df_RandomForest.to_csv('RandomForest.csv', index=False)

df_xgboost = pd.DataFrame(model_xgboost.cv_results_)
df_xgboost.to_csv('xgboost.csv', index=False)

df_lightgbm = pd.DataFrame(model_lightgbm.cv_results_)
df_lightgbm.to_csv('lightgbm.csv', index=False)

df_catboost = pd.DataFrame(model_catbootst.cv_results_)
df_catboost.to_csv('catboost.csv', index=False)



o_RandomForest = model_RandomForest.predict(X_test)
o_xgboost = model_xgboost.predict(X_test)
o_lightgbm = model_lightgbm.predict(X_test)
o_catboost = model_catbootst.predict(X_test)


metric_RandomForest = mean_squared_error(y_test, o_RandomForest, squared = False)
metric_xgboost = mean_squared_error(y_test, o_xgboost, squared = False)
metric_lightgbm = mean_squared_error(y_test, o_lightgbm, squared = False)
metric_catboost = mean_squared_error(y_test, o_catboost, squared = False)

print("RandomForest: ", metric_RandomForest)
print("xgboost: ", metric_xgboost)
print("lightgbm: ", metric_lightgbm)
print("catboost: ", metric_catboost)