from prediccion_utils import read_dataset, process_predictions
from sklearn.ensemble import IsolationForest
from skopt import BayesSearchCV
from sklearn.ensemble import RandomForestRegressor
import numpy as np
import pandas as pd
from sklearn.metrics import mean_squared_error
from sklearn.neural_network import MLPRegressor
from sklearn.tree import plot_tree
import matplotlib.pyplot as plt
from time import time


from xgboost import XGBRegressor
from lightgbm import LGBMRegressor
from catboost import CatBoostRegressor

#########################################
### 1. CARGA DE DATOS Y PREPROCESAMIENTO
#########################################
# Lectura de datos que carga el conjunto de train y test 

df_total, df_train, df_test = read_dataset("../data/UH_2023_TRAIN.txt")


df_train = df_train[["SUPERFICIE","MODO","VARIEDAD_32","VARIEDAD_87", "VARIEDAD_17",
"ALTITUD_MAX","ALTITUD_MIN","PRODUCCION","PRODUCCION_MEDIA"]]

df_test_aux = df_test[["SUPERFICIE","MODO","VARIEDAD_32","VARIEDAD_87", "VARIEDAD_17",
"ALTITUD_MAX","ALTITUD_MIN","PRODUCCION","PRODUCCION_MEDIA"]]

# Mantenemos unicamente las filas con superficie > 0
df_train = df_train[df_train["SUPERFICIE"] > 0]

# Obtencion manual de las fincas con superficie 0
df_22 = df_total[df_total["CAMPAÑA"] == 22].drop(["PRODUCCION"], axis=1)
df_calc = df_total[df_total["CAMPAÑA"] < 22].drop(["CAMPAÑA","SUPERFICIE"], axis=1)
df_calc = df_calc.groupby(["ID_FINCA","VARIEDAD","MODO","TIPO","COLOR"]).mean().reset_index()
df_manual = df_22[df_22["SUPERFICIE"] == 0].merge(df_calc, on=["ID_FINCA","VARIEDAD","MODO","TIPO","COLOR"], how="left")

print(df_manual[['ID_FINCA', 'VARIEDAD', 'MODO', 'TIPO', 'COLOR', 'SUPERFICIE', 'PRODUCCION']])

#########################################
### 2. ENTRENAMIENTO DEL MODELO
#########################################

y = df_train.pop("PRODUCCION")
X = df_train.values

df_test = df_test[df_test["SUPERFICIE"] > 0]
df_test_aux = df_test_aux[df_test_aux["SUPERFICIE"] > 0]

y_test = df_test_aux.pop("PRODUCCION")
X_test = df_test_aux.values

model = RandomForestRegressor(
                        bootstrap=True,
                        max_depth=80,
                        max_features='sqrt',
                        min_samples_leaf=1,
                        min_samples_split=5,
                        n_estimators=1000)


train_time_0 = time()
model.fit(X,y)
train_time = time() - train_time_0


#########################################
### 3. GENERACIÓN DE PREDICCIONES
#########################################

test_time_0 = time()
o = model.predict(X_test)
test_time = time() - test_time_0

print("Train time: ", train_time)
print("Test time: ", test_time)


res = process_predictions(o, df_test, df_manual)


res.to_csv("UH2023_Universidad de Sevilla (US)_Los Palmerines.txt",index=False, sep="|", header=False)