import pandas as pd
import numpy as np
from functools import reduce

def altura_max(a):
    if "-" in str(a):
        aux = str(a).split("-")
        return float(aux[1])
    elif str(a) == "nan":
        return "nan"
    else:
        return a
    
def altura_min(a):
    if "-" in str(a):
        aux = str(a).split("-")
        return float(aux[0])
    elif str(a) == "nan":
        return "nan"
    else:
        return a



def process_altura(df):
    df["ALTITUD_MAX"] = df["ALTITUD"].apply(altura_max)
    df["ALTITUD_MIN"] = df["ALTITUD"].apply(altura_min)

    # Eliminamos columna altitud y asignamos el tipo correcto a las nuevas variables
    df.drop(columns=["ALTITUD"],axis=1, inplace=True)
    df = df.astype({"ALTITUD_MAX": float, "ALTITUD_MIN": float})

    # Sustituimos ahora los valores "NA" por la media de la altura de su estacion correspondiente
    for i in range(20):

        subset = df[(df['ID_ESTACION'] == i) & (df["ALTITUD_MAX"] != "nan")]

        mean_max = subset["ALTITUD_MAX"].mean()
        mean_min = subset["ALTITUD_MIN"].mean()

        df['ALTITUD_MAX'][df['ID_ESTACION'] == i] = df['ALTITUD_MAX'][df['ID_ESTACION'] == i].fillna(mean_max)
        df['ALTITUD_MIN'][df['ID_ESTACION'] == i] = df['ALTITUD_MIN'][df['ID_ESTACION'] == i].fillna(mean_min)

    return df


def process_superficie(df):

    df_mayor_2020 = df[df["CAMPAÑA"] >= 20]
    df_menor_2020 = df[df["CAMPAÑA"] < 20]

    df_aux = df_mayor_2020[df_mayor_2020["SUPERFICIE"] > 0]
    df_menor_2020.drop("SUPERFICIE", axis=1, inplace=True)
    

    df_aux = df_aux.drop("CAMPAÑA", axis=1).drop_duplicates()
    df_aux = df_aux.groupby(['ID_FINCA','VARIEDAD','MODO', "TIPO", "COLOR"]).mean().reset_index()
    df_aux = df_aux[['ID_FINCA','VARIEDAD','MODO', "TIPO", "COLOR","SUPERFICIE"]]


    df_menor_2020 = df_menor_2020.merge(df_aux, on=['ID_FINCA','VARIEDAD','MODO', "TIPO", "COLOR"], how='left')


    df = pd.concat([df_menor_2020, df_mayor_2020])
    df.fillna(0, inplace=True)

    return df


def process_one_hot(df):
    one_hot = pd.get_dummies(df.VARIEDAD, prefix='VARIEDAD')
    df = df.join(one_hot)
    df['MODO'] = df['MODO'].replace([2], 0)

    return df


def mean_of_first_n(row):
      return sum(row["PRODUCCION"][max(row["CAMPAÑA_INDEX"]-2,0):row["CAMPAÑA_INDEX"]])/min(row["CAMPAÑA_INDEX"],2)

def process_produccion_media(df):
    df_algo = df.groupby(['ID_FINCA', "VARIEDAD", "MODO", "TIPO", "COLOR"]).agg({'CAMPAÑA': lambda x: list(x),'PRODUCCION': lambda x: list(x),}).reset_index()

    df_algo['CAMPAÑA'] = df_algo['CAMPAÑA'].apply(lambda x: x[1:])
    df_algo['PRODUCCION'] = df_algo['PRODUCCION'].apply(lambda x: x[:-1])
    df_algo.drop(index=df_algo[df_algo['CAMPAÑA'].apply(lambda x: len(x)==0)].index, inplace=True)
    df_algo = df_algo.explode(["CAMPAÑA"]).reset_index()
    df_algo['CAMPAÑA_INDEX'] = df_algo.groupby('index').cumcount() + 1
    df_algo["PRODUCCION_MEDIA"] = df_algo.apply(mean_of_first_n, axis=1)
    df_algo = df_algo.drop(["PRODUCCION","CAMPAÑA_INDEX","index"], axis=1)

    df_res = pd.merge(df,df_algo,on=["ID_FINCA","VARIEDAD","MODO","TIPO","COLOR","CAMPAÑA"],how='left')

    mean_A = df_res.loc[df_res["CAMPAÑA"] == 22, "PRODUCCION_MEDIA"].mean()
    df_res.loc[df_res["CAMPAÑA"] == 22, "PRODUCCION_MEDIA"] = df_res.loc[df_res["CAMPAÑA"] == 22, "PRODUCCION_MEDIA"].fillna(mean_A)

    return df_res


def process_date(df):
    df_split_date = df[[i for i in df.columns]]
    df_split_date.insert(0, "date", df.validTimeUtc.str.split(" ", expand = True)[0])
    df_split_date.insert(1, "year", df_split_date.date.str.split("-", expand = True)[0])
    df_split_date.insert(2, "month", df_split_date.date.str.split("-", expand = True)[1])
    df_split_date.insert(3, "day", df_split_date.date.str.split("-", expand = True)[2])
    df_split_date.insert(4, "hour", df_split_date.validTimeUtc.str.split(" ", expand = True)[1])
    df_split_date = df_split_date.drop('validTimeUtc', axis=1)
    df_split_date = df_split_date.drop('date', axis=1)
    df_split_date = df_split_date[(df_split_date['month'] != "07") & (df_split_date['month'] != "08") 
                              & (df_split_date['month'] != "09") & (df_split_date['month'] != "10") 
                              & (df_split_date['month'] != "11") & (df_split_date['month'] != "12") 
                              & (df_split_date['month'] != "01") & (df_split_date['month'] != "02") 
                              & (df_split_date['year'] != "2015")]
    df_split_date = df_split_date[['year', 'month', 'day', 'hour', 'precip1Hour', 'relativeHumidity', 
                            'temperature', 'temperatureMax24Hour', 'temperatureMin24Hour', 'windSpeed', 'ID_ESTACION']]
    return df_split_date


def gsp(df_columns):
    df_precip = df_columns[['precip1Hour', 'ID_ESTACION', 'year']].groupby(['ID_ESTACION', 'year']).mean().reset_index()

    df_precip.rename(columns = {"precip1Hour" : "GSP"}, inplace=True)

    return df_precip

def gdd(df_columns):
    ref = df_columns["temperature"].mean()

    df_temp = df_columns[['ID_ESTACION', 'year', 'month', 'day', 'temperature']].groupby(
                            ['ID_ESTACION', 'year', 'month', 'day']).mean().reset_index()

    df_temp['GDD'] = df_temp['temperature'] - ref

    df_GDD_season = df_temp[['ID_ESTACION', 'year', 'month', 'day', 'GDD']].groupby(['ID_ESTACION', 'year']).sum().reset_index()
    
    return df_GDD_season


def tMin(df_columns):
    df_t_min = df_columns[["ID_ESTACION", "year", "month", "day", "temperature"]].groupby(
                            ['ID_ESTACION','year', 'month', 'day']).min().reset_index()

    df_t_min_year = df_t_min[['ID_ESTACION','year','month','day', 'temperature']].groupby(
                            ['ID_ESTACION', 'year']).mean().reset_index()

    df_t_min_year.rename(columns = {"temperature" : "tMin"}, inplace=True)

    return df_t_min_year

def tMax(df_columns):
    df_t_max = df_columns[["ID_ESTACION", "year", "month", "day", "temperature"]].groupby(
                        ['ID_ESTACION', 'year', 'month', 'day']).max().reset_index()

    df_t_max_year = df_t_max[['ID_ESTACION','year','month','day', 'temperature']].groupby(
                            ['ID_ESTACION', 'year']).mean().reset_index()

    df_t_max_year.rename(columns = {"temperature" : "tMax"}, inplace=True)

    return df_t_max_year



def frostDays(df_t_min):
    df_t_min['frostDays'] = df_t_min['temperature'].apply(lambda x: 1 if x<0 else 0)

    df_year_frost = df_t_min[['ID_ESTACION', 'year', 'frostDays']].groupby(['ID_ESTACION', 'year']).sum().reset_index()

    return df_year_frost    



# summerDays

def summerDays(df_t_max):
    df_t_max['summerDays'] = df_t_max['temperature'].apply(lambda x: 1 if x>25 else 0)

    df_year_summer = df_t_max[['ID_ESTACION', 'year', 'summerDays']].groupby(['ID_ESTACION', 'year']).sum().reset_index()

    return df_year_summer





# growthStoppedDays

def growthStoppedDays(df_t_min):
    df_t_min['growthStopped'] = df_t_min['temperature'].apply(lambda x: 1 if x<7 else 0)

    df_year_stops = df_t_min[['ID_ESTACION', 'year', 'growthStopped']].groupby(['ID_ESTACION', 'year']).sum().reset_index()

    return df_year_stops





# damagedVineDays

def damagedVineDays(df_t_max):
    df_t_max['damagedVine'] = df_t_max['temperature'].apply(lambda x: 1 if x>=35 else 0)

    df_year_damaged = df_t_max[['ID_ESTACION', 'year', 'damagedVine']].groupby(['ID_ESTACION', 'year']).sum().reset_index()

    return df_year_damaged





# longestDrySpell

def longestDrySpell(df_columns):
    df_longests = df_columns[['ID_ESTACION', 'year', 'month', 'day', 'precip1Hour']].groupby(
                            ['ID_ESTACION', 'year', 'month', 'day']).sum().reset_index()

    df_longests['longestDrySpell'] = df_longests['precip1Hour'].apply(lambda x: 1 if x<1 else 0)

    df_longests['accumLongestDrySpell'] = df_longests.groupby(
        (df_longests['longestDrySpell'] != df_longests['longestDrySpell'].shift()).cumsum())['longestDrySpell'].cumsum()

    df_dry_spell = df_longests.groupby(['ID_ESTACION', 'year'])['accumLongestDrySpell'].max().reset_index()

    return df_dry_spell




# longestWetSpell

def longestWetSpell(df_columns):
    df_longests = df_columns[['ID_ESTACION', 'year', 'month', 'day', 'precip1Hour']].groupby(
                            ['ID_ESTACION', 'year', 'month', 'day']).sum().reset_index()

    df_longests['longestWetSpell'] = df_longests['precip1Hour'].apply(lambda x: 1 if x>=1 else 0)

    df_longests['accumLongestWetSpell'] = df_longests.groupby(
        (df_longests['longestWetSpell'] != df_longests['longestWetSpell'].shift()).cumsum())['longestWetSpell'].cumsum()

    df_wet_spell = df_longests.groupby(['ID_ESTACION', 'year'])['accumLongestWetSpell'].max().reset_index()

    return df_wet_spell


def read_dataset(route):
    df = pd.read_csv(route, sep="|")
    df = process_altura(df)
    df = process_superficie(df)
    df = process_one_hot(df)
    df = process_produccion_media(df)


    df_meteo = pd.read_csv("../data/DATOS_METEO.TXT", delimiter='|')
    df_columns = process_date(df_meteo)

    df_precip = gsp(df_columns)
    df_year = df_precip[['ID_ESTACION', 'year', 'GSP']]


    df_GDD_season = gdd(df_columns)
    df_year.insert(3, "GDD", df_GDD_season["GDD"])

    df_t_min_year = tMin(df_columns)
    df_year.insert(4, "tMin", df_t_min_year["tMin"])
    df_t_max_year = tMax(df_columns)
    df_year.insert(5, "tMax", df_t_max_year["tMax"])


    df_t_min = df_columns[["ID_ESTACION", "year", "month", "day", "temperature"]].groupby(
                        ['ID_ESTACION','year', 'month', 'day']).min().reset_index()
    df_year_frost = frostDays(df_t_min)
    df_year.insert(6, "frostDays", df_year_frost["frostDays"])

    df_t_max = df_columns[["ID_ESTACION", "year", "month", "day", "temperature"]].groupby(
                        ['ID_ESTACION', 'year', 'month', 'day']).max().reset_index()
    df_year_summer = summerDays(df_t_max)
    df_year.insert(7, "summerDays", df_year_summer["summerDays"])

    df_year_stops = growthStoppedDays(df_t_min)
    df_year.insert(8, "growthStoppedDays", df_year_stops["growthStopped"])

    df_year_damaged = damagedVineDays(df_t_max)
    df_year.insert(9, "damagedVine", df_year_damaged["damagedVine"])
    df_year.rename(columns = {"damagedVine" : "damagedVineDays"}, inplace=True)

    df_dry_spell = longestDrySpell(df_columns)
    df_year.insert(10, "longestDrySpell", df_dry_spell['accumLongestDrySpell'])

    df_wet_spell = longestWetSpell(df_columns)
    df_year.insert(11, "longestWetSpell", df_wet_spell['accumLongestWetSpell'])


    df_year['CAMPAÑA'] = df_year['year'].apply(lambda x: int(str(x)[-2:]))
    df_year = df_year.drop("year", axis=1)
    df_final = pd.merge(df,df_year,on=["ID_ESTACION","CAMPAÑA"],how='left').dropna()


    df_train = df_final[df_final["CAMPAÑA"] < 22]
    df_test = df_final[df_final["CAMPAÑA"] == 22] 


    df_final.to_csv("UH_2023_pruebas.csv", index=False)

    return df_final, df_train, df_test

    
def process_predictions(o, df_test, df_manual):
    df_test["PRODUCCION"] = o

    # Concatenamos las predicciones con las manuales
    df_test = pd.concat([df_test, df_manual], axis=0, ignore_index=True, sort=False, join='outer')

    # Deshacemos los cambios en modo
    df_test['MODO'] = df_test['MODO'].replace([0], 2)

    # Redondeamos a 2 posiciones decimales
    df_test = df_test.round(2)

    # Ordenamos en el orden indicado
    df_test.sort_values(by=["ID_FINCA", "VARIEDAD", "MODO", "TIPO", "COLOR", "SUPERFICIE"], inplace = True)

    # Mantenemos unicamente las columnas indicadas
    df_test = df_test[["ID_FINCA", "VARIEDAD", "MODO", "TIPO", "COLOR", "SUPERFICIE", "PRODUCCION"]]

    return df_test


